@@ -116,7 +116,7 @@ class BaseBackend(object):
                                  status=states.SUCCESS, request=request)
 
     def mark_as_failure(self, task_id, exc, traceback=None, request=None):
-        """Mark task as executed with failure. Stores the execption."""
+        """Mark task as executed with failure. Stores the exception."""
         return self.store_result(task_id, exc, status=states.FAILURE,
                                  traceback=traceback, request=request)
 
@@ -166,11 +166,11 @@ class BaseBackend(object):
     def exception_to_python(self, exc):
         """Convert serialized exception to Python exception."""
         if exc:
-            if self.serializer in EXCEPTION_ABLE_CODECS:
-                return get_pickled_exception(exc)
-            elif not isinstance(exc, BaseException):
-                return create_exception_cls(
+            if not isinstance(exc, BaseException):
+                exc = create_exception_cls(
                     from_utf8(exc['exc_type']), __name__)(exc['exc_message'])
+            if self.serializer in EXCEPTION_ABLE_CODECS:
+                exc = get_pickled_exception(exc)
         return exc
 
     def prepare_value(self, result):
@@ -241,6 +241,8 @@ class BaseBackend(object):
         return self.persistent if p is None else p
 
     def encode_result(self, result, status):
+        if isinstance(result, ExceptionInfo):
+            result = result.exception
         if status in self.EXCEPTION_STATES and isinstance(result, Exception):
             return self.prepare_exception(result)
         else:
